@@ -14,7 +14,6 @@ use crate::core::Error;
 use crate::core::Result;
 use crate::executor::{ExecutorMut, RBatisConnExecutor, RBatisTxExecutor};
 use crate::plugin::page::{IPageRequest, Page, IPage};
-use crate::plugin::version_lock::VersionLockPlugin;
 use crate::rbatis::Rbatis;
 use crate::sql::rule::SqlRule;
 use crate::utils::string_util::to_snake_name;
@@ -22,6 +21,8 @@ use crate::wrapper::Wrapper;
 use std::marker::PhantomData;
 use std::ops::{Deref, DerefMut};
 use serde_json::value::Value::Null;
+use std::option::Option::Some;
+
 
 /// DataBase Table Model trait
 ///
@@ -30,10 +31,7 @@ use serde_json::value::Value::Null;
 ///  #[crud_table(table_name:"biz_activity"|table_columns:"id,name,version,delete_flag"|formats_pg:"id:{}::uuid")]
 ///
 ///
-pub trait CRUDTable: Send + Sync + Serialize + DeserializeOwned {
-    /// is enable use plugin
-    fn is_use_plugin(plugin_name: &str) -> bool { true }
-
+pub trait CRUDTable: Send + Sync + Serialize {
     /// get table name,default is type name for snake name
     ///
     /// for Example:  struct  BizActivity{} =>  "biz_activity"
@@ -63,27 +61,7 @@ pub trait CRUDTable: Send + Sync + Serialize + DeserializeOwned {
     /// you also can impl this method for static string
     ///
     /// If a macro is used, the method is overridden by the macro
-    fn table_columns() -> String {
-        let bean: serde_json::Result<Self> = serde_json::from_str("{}");
-        if bean.is_err() {
-            //if json decode fail,return '*'
-            return " * ".to_string();
-        }
-        let v = json!(&bean.unwrap());
-        if !v.is_object() {
-            //if json decode fail,return '*'
-            return " * ".to_string();
-        }
-        let m = v.as_object().unwrap();
-        let mut fields = String::new();
-        for (k, _) in m {
-            fields.push_str(k);
-            fields.push_str(",");
-        }
-        fields.pop();
-        return format!("{}", fields);
-    }
-
+    fn table_columns() -> String;
 
     ///format column
     fn do_format_column(driver_type: &DriverType, column: &str, data: &mut String) {
@@ -190,6 +168,49 @@ pub trait CRUDTable: Send + Sync + Serialize + DeserializeOwned {
     }
 }
 
+/// decode columns from json and type
+pub fn decode_table_columns<T>() -> String where T: DeserializeOwned + Serialize {
+    let bean: serde_json::Result<T> = serde_json::from_str("{}");
+    if bean.is_err() {
+        //if json decode fail,return '*'
+        return " * ".to_string();
+    }
+    let v = json!(&bean.unwrap());
+    if !v.is_object() {
+        //if json decode fail,return '*'
+        return " * ".to_string();
+    }
+    let m = v.as_object().unwrap();
+    let mut fields = String::new();
+    for (k, _) in m {
+        fields.push_str(k);
+        fields.push_str(",");
+    }
+    fields.pop();
+    return format!("{}", fields);
+}
+
+
+impl<T> CRUDTable for &T where T: CRUDTable {
+    fn table_name() -> String {
+        T::table_name()
+    }
+
+    fn table_columns() -> String {
+        T::table_columns()
+    }
+}
+
+impl<T> CRUDTable for &mut T where T: CRUDTable {
+    fn table_name() -> String {
+        T::table_name()
+    }
+
+    fn table_columns() -> String {
+        T::table_columns()
+    }
+}
+
 impl<T> CRUDTable for Option<T>
     where
         T: CRUDTable,
@@ -281,48 +302,48 @@ pub trait CRUD {
 
     /// remove_by_column
     /// column_value,column's value
-    async fn remove_by_column<T, C>(&self, column: &str, column_value: &C) -> Result<u64> where T: CRUDTable, C: Serialize + Send + Sync;
+    async fn remove_by_column<T, P>(&self, column: &str, column_value: &P) -> Result<u64> where T: CRUDTable, P: Serialize + Send + Sync;
 
     /// remove_batch_by_column
     /// column_values,column's value
-    async fn remove_batch_by_column<T, C>(&self, column: &str, column_values: &[C]) -> Result<u64>
+    async fn remove_batch_by_column<T, P>(&self, column: &str, column_values: &[P]) -> Result<u64>
         where
-            T: CRUDTable, C: Serialize + Send + Sync;
+            T: CRUDTable, P: Serialize + Send + Sync;
 
     /// update_by_wrapper
     /// skips: use &[Skip::Null] will skip id column and null value param
     async fn update_by_wrapper<T>(
         &self,
-        table: &mut T,
+        table: &T,
         w: &Wrapper,
         skips: &[Skip],
     ) -> Result<u64>
         where
             T: CRUDTable;
     /// update database record by id
-    async fn update_by_column<T>(&self, column: &str, table: &mut T) -> Result<u64>
+    async fn update_by_column<T>(&self, column: &str, table: &T) -> Result<u64>
         where
             T: CRUDTable;
 
     /// remove batch database record by args
-    async fn update_batch_by_column<T>(&self, column: &str, tables: &mut [T]) -> Result<u64>
+    async fn update_batch_by_column<T>(&self, column: &str, tables: &[T]) -> Result<u64>
         where
             T: CRUDTable;
 
     /// fetch database record by id
-    async fn fetch_by_column<T, C>(&self, column: &str, value: &C) -> Result<T>
+    async fn fetch_by_column<T, P>(&self, column: &str, value: &P) -> Result<T>
         where
-            T: CRUDTable, C: Serialize + Send + Sync;
+            T: CRUDTable + DeserializeOwned, P: Serialize + Send + Sync;
 
     /// fetch database record by a wrapper
     async fn fetch_by_wrapper<T>(&self, w: &Wrapper) -> Result<T>
         where
-            T: CRUDTable;
+            T: CRUDTable + DeserializeOwned;
 
     /// count database record by a wrapper
     async fn fetch_count_by_wrapper<T>(&self, w: &Wrapper) -> Result<u64>
         where
-            T: CRUDTable;
+            T: CRUDTable + DeserializeOwned;
 
     /// fetch page database record list by a wrapper
     async fn fetch_page_by_wrapper<T>(
@@ -331,22 +352,22 @@ pub trait CRUD {
         page: &dyn IPageRequest,
     ) -> Result<Page<T>>
         where
-            T: CRUDTable;
+            T: CRUDTable + DeserializeOwned;
 
     /// fetch database record list for all
     async fn fetch_list<T>(&self) -> Result<Vec<T>>
         where
-            T: CRUDTable;
+            T: CRUDTable + DeserializeOwned;
 
     /// fetch database record list by a id array
-    async fn fetch_list_by_column<T, C>(&self, column: &str, column_values: &[C]) -> Result<Vec<T>>
+    async fn fetch_list_by_column<T, P>(&self, column: &str, column_values: &[P]) -> Result<Vec<T>>
         where
-            T: CRUDTable, C: Serialize + Send + Sync;
+            T: CRUDTable + DeserializeOwned, P: Serialize + Send + Sync;
 
     /// fetch database record list by a wrapper
     async fn fetch_list_by_wrapper<T>(&self, w: &Wrapper) -> Result<Vec<T>>
         where
-            T: CRUDTable;
+            T: CRUDTable + DeserializeOwned;
 
     /// fetch page result(prepare sql)
     async fn fetch_page<T>(
@@ -500,14 +521,18 @@ pub trait CRUDMut: ExecutorMut {
         let table_name = choose_dyn_table_name::<T>(w);
         let where_sql = self.driver_type()?.make_where(&w.sql);
         let mut sql = String::new();
-        if self.get_rbatis().logic_plugin.is_some() && T::is_use_plugin(self.get_rbatis().logic_plugin.as_ref().unwrap().name()) {
-            sql = self.get_rbatis().logic_plugin.as_ref().unwrap().create_remove_sql(
-                &self.driver_type()?,
-                &table_name,
-                &T::table_columns(),
-                &where_sql,
-            )?;
-        } else {
+
+        if let Some(logic) = &self.get_rbatis().logic_plugin {
+            if w.dml.eq("where") {
+                sql = logic.create_remove_sql(
+                    &self.driver_type()?,
+                    &table_name,
+                    &T::table_columns(),
+                    &where_sql,
+                )?;
+            }
+        }
+        if sql.is_empty() {
             sql = format!(
                 "{} {} {}",
                 crate::sql::TEMPLATE.delete_from.value,
@@ -522,17 +547,17 @@ pub trait CRUDMut: ExecutorMut {
     }
 
     /// remove database record by id
-    async fn remove_by_column<T, C>(&mut self, column: &str, value: &C) -> Result<u64>
+    async fn remove_by_column<T, P>(&mut self, column: &str, value: &P) -> Result<u64>
         where
-            T: CRUDTable, C: Serialize + Send + Sync,
+            T: CRUDTable, P: Serialize + Send + Sync,
     {
         let mut sql = String::new();
         let driver_type = &self.driver_type()?;
         let mut data = String::new();
         driver_type.stmt_convert(0, &mut data);
         T::do_format_column(&driver_type, column, &mut data);
-        if self.get_rbatis().logic_plugin.is_some() && T::is_use_plugin(self.get_rbatis().logic_plugin.as_ref().unwrap().name()) {
-            sql = self.get_rbatis().logic_plugin.as_ref().unwrap().create_remove_sql(
+        if let Some(logic) = &self.get_rbatis().logic_plugin {
+            sql = logic.create_remove_sql(
                 &driver_type,
                 T::table_name().as_str(),
                 &T::table_columns(),
@@ -541,10 +566,10 @@ pub trait CRUDMut: ExecutorMut {
                     crate::sql::TEMPLATE.r#where.value,
                     column,
                     data
-                )
-                    .as_str(),
+                ).as_str(),
             )?;
-        } else {
+        }
+        if sql.is_empty() {
             sql = format!(
                 "{} {} {} {} = {}",
                 crate::sql::TEMPLATE.delete_from.value,
@@ -565,9 +590,9 @@ pub trait CRUDMut: ExecutorMut {
     /// rb.remove_batch_by_column::<BizActivity>(&["1".to_string(),"2".to_string()]).await;
     /// [rbatis] Exec ==> delete from biz_activity where id IN ( ? , ? )
     ///
-    async fn remove_batch_by_column<T, C>(&mut self, column: &str, values: &[C]) -> Result<u64>
+    async fn remove_batch_by_column<T, P>(&mut self, column: &str, values: &[P]) -> Result<u64>
         where
-            T: CRUDTable, C: Serialize + Send + Sync
+            T: CRUDTable, P: Serialize + Send + Sync
     {
         if values.is_empty() {
             return Ok(0);
@@ -584,7 +609,7 @@ pub trait CRUDMut: ExecutorMut {
     /// skips: use &[Skip::Value(&serde_json::Value::Null), Skip::Column("id"), Skip::Column(column)] will skip id column and null value param
     async fn update_by_wrapper<T>(
         &mut self,
-        table: &mut T,
+        table: &T,
         w: &Wrapper,
         skips: &[Skip],
     ) -> Result<u64>
@@ -608,7 +633,6 @@ pub trait CRUDMut: ExecutorMut {
         }
         let null = serde_json::Value::Null;
         let mut sets = String::new();
-
         for column in columns_vec {
             //filter
             let mut is_continue = false;
@@ -657,15 +681,6 @@ pub trait CRUDMut: ExecutorMut {
                     data
                 ).as_str(),
             );
-            match &self.get_rbatis().version_lock_plugin {
-                Some(version_lock_plugin) => {
-                    if T::is_use_plugin(self.get_rbatis().version_lock_plugin.as_ref().unwrap().name()) {
-                        old_version = v.clone();
-                        v = version_lock_plugin.try_add_one(&old_version, column);
-                    }
-                }
-                _ => {}
-            }
             args.push(v.clone());
         }
         sets.pop();
@@ -678,30 +693,6 @@ pub trait CRUDMut: ExecutorMut {
             sets
         );
         wrapper.args = args;
-
-
-        //version lock
-        match self.get_rbatis().version_lock_plugin.as_ref() {
-            Some(version_lock_plugin) => {
-                if T::is_use_plugin(self.get_rbatis().version_lock_plugin.as_ref().unwrap().name()) {
-                    let version_sql = version_lock_plugin
-                        .as_ref()
-                        .try_make_where_sql(&old_version);
-                    if !version_sql.is_empty() {
-                        if !wrapper
-                            .sql
-                            .contains(crate::sql::TEMPLATE.r#where.left_right_space)
-                        {
-                            wrapper
-                                .sql
-                                .push_str(crate::sql::TEMPLATE.r#where.left_right_space);
-                        }
-                        wrapper.sql.push_str(&version_sql);
-                    }
-                }
-            }
-            _ => {}
-        }
         if !w.sql.is_empty() {
             if !wrapper
                 .sql
@@ -714,20 +705,16 @@ pub trait CRUDMut: ExecutorMut {
             wrapper = wrapper.and();
             wrapper = wrapper.push_wrapper(&w);
         }
-
         let rows_affected = self
             .exec(wrapper.sql.as_str(), &wrapper.args)
             .await?
             .rows_affected;
-        if rows_affected > 0 {
-            *table = serde_json::from_value(serde_json::Value::Object(map)).into_result()?;
-        }
         return Ok(rows_affected);
     }
 
     /// update database record by id
     /// update sql will be skip null value and id column
-    async fn update_by_column<T>(&mut self, column: &str, table: &mut T) -> Result<u64>
+    async fn update_by_column<T>(&mut self, column: &str, table: &T) -> Result<u64>
         where
             T: CRUDTable
     {
@@ -745,7 +732,7 @@ pub trait CRUDMut: ExecutorMut {
     }
 
     /// remove batch database record by args
-    async fn update_batch_by_column<T>(&mut self, column: &str, args: &mut [T]) -> Result<u64>
+    async fn update_batch_by_column<T>(&mut self, column: &str, args: &[T]) -> Result<u64>
         where
             T: CRUDTable
     {
@@ -759,7 +746,7 @@ pub trait CRUDMut: ExecutorMut {
     /// fetch database record by a wrapper
     async fn fetch_by_wrapper<T>(&mut self, w: &Wrapper) -> Result<T>
         where
-            T: CRUDTable,
+            T: CRUDTable + DeserializeOwned,
     {
         let sql = make_select_sql::<T>(self.get_rbatis(), &T::table_columns(), &w)?;
         return self.fetch(sql.as_str(), &w.args).await;
@@ -775,9 +762,9 @@ pub trait CRUDMut: ExecutorMut {
     }
 
     /// fetch database record by value
-    async fn fetch_by_column<T, C>(&mut self, column: &str, value: &C) -> Result<T>
+    async fn fetch_by_column<T, P>(&mut self, column: &str, value: &P) -> Result<T>
         where
-            T: CRUDTable, C: Serialize + Send + Sync,
+            T: CRUDTable + DeserializeOwned, P: Serialize + Send + Sync,
     {
         let w = self.get_rbatis().new_wrapper_table::<T>().eq(&column, value);
         return self.fetch_by_wrapper(&w).await;
@@ -786,7 +773,7 @@ pub trait CRUDMut: ExecutorMut {
     /// fetch database record list by a wrapper
     async fn fetch_list_by_wrapper<T>(&mut self, w: &Wrapper) -> Result<Vec<T>>
         where
-            T: CRUDTable,
+            T: CRUDTable + DeserializeOwned,
     {
         let sql = make_select_sql::<T>(self.get_rbatis(), &T::table_columns(), &w)?;
         return self.fetch(sql.as_str(), &w.args).await;
@@ -795,7 +782,7 @@ pub trait CRUDMut: ExecutorMut {
     /// fetch database record list for all
     async fn fetch_list<T>(&mut self) -> Result<Vec<T>>
         where
-            T: CRUDTable,
+            T: CRUDTable + DeserializeOwned,
     {
         let rb = self.get_rbatis();
         return self.fetch_list_by_wrapper(&rb.new_wrapper_table::<T>())
@@ -803,9 +790,9 @@ pub trait CRUDMut: ExecutorMut {
     }
 
     /// fetch database record list by a id array
-    async fn fetch_list_by_column<T, C>(&mut self, column: &str, column_values: &[C]) -> Result<Vec<T>>
+    async fn fetch_list_by_column<T, P>(&mut self, column: &str, column_values: &[P]) -> Result<Vec<T>>
         where
-            T: CRUDTable, C: Serialize + Send + Sync,
+            T: CRUDTable + DeserializeOwned, P: Serialize + Send + Sync,
     {
         if column_values.is_empty() {
             return Ok(vec![]);
@@ -821,7 +808,7 @@ pub trait CRUDMut: ExecutorMut {
         page: &dyn IPageRequest,
     ) -> Result<Page<T>>
         where
-            T: CRUDTable,
+            T: CRUDTable + DeserializeOwned,
     {
         let sql = make_select_sql::<T>(self.get_rbatis(), &T::table_columns(), &w)?;
         self.fetch_page(sql.as_str(), &w.args, page).await
@@ -934,79 +921,79 @@ impl CRUD for Rbatis {
         conn.remove_by_wrapper::<T>(w).await
     }
 
-    async fn remove_by_column<T, C>(&self, column: &str, value: &C) -> Result<u64> where
-        T: CRUDTable, C: Serialize + Send + Sync {
+    async fn remove_by_column<T, P>(&self, column: &str, value: &P) -> Result<u64> where
+        T: CRUDTable, P: Serialize + Send + Sync {
         let mut conn = self.acquire().await?;
-        conn.remove_by_column::<T, C>(column, value).await
+        conn.remove_by_column::<T, P>(column, value).await
     }
 
-    async fn remove_batch_by_column<T, C>(&self, column: &str, values: &[C]) -> Result<u64> where
-        T: CRUDTable, C: Serialize + Send + Sync {
+    async fn remove_batch_by_column<T, P>(&self, column: &str, values: &[P]) -> Result<u64> where
+        T: CRUDTable, P: Serialize + Send + Sync {
         let mut conn = self.acquire().await?;
-        conn.remove_batch_by_column::<T, C>(column, values).await
+        conn.remove_batch_by_column::<T, P>(column, values).await
     }
 
     /// update_by_wrapper
     /// skips: use &[Skip::Value(&serde_json::Value::Null), Skip::Column("id"), Skip::Column(column)] will skip id column and null value param
-    async fn update_by_wrapper<T>(&self, table: &mut T, w: &Wrapper, skips: &[Skip]) -> Result<u64> where
+    async fn update_by_wrapper<T>(&self, table: &T, w: &Wrapper, skips: &[Skip]) -> Result<u64> where
         T: CRUDTable {
         let mut conn = self.acquire().await?;
         conn.update_by_wrapper(table, w, skips).await
     }
 
-    async fn update_by_column<T>(&self, column: &str, table: &mut T) -> Result<u64> where
+    async fn update_by_column<T>(&self, column: &str, table: &T) -> Result<u64> where
         T: CRUDTable {
         let mut conn = self.acquire().await?;
         conn.update_by_column(column, table).await
     }
 
-    async fn update_batch_by_column<T>(&self, column: &str, args: &mut [T]) -> Result<u64> where
+    async fn update_batch_by_column<T>(&self, column: &str, args: &[T]) -> Result<u64> where
         T: CRUDTable {
         let mut conn = self.acquire().await?;
         conn.update_batch_by_column::<T>(column, args).await
     }
 
-    async fn fetch_by_column<T, C>(&self, column: &str, value: &C) -> Result<T> where
-        T: CRUDTable, C: Serialize + Send + Sync {
+    async fn fetch_by_column<T, P>(&self, column: &str, value: &P) -> Result<T> where
+        T: CRUDTable + DeserializeOwned, P: Serialize + Send + Sync {
         let mut conn = self.acquire().await?;
-        conn.fetch_by_column::<T, C>(column, value).await
+        conn.fetch_by_column::<T, P>(column, value).await
     }
 
     async fn fetch_by_wrapper<T>(&self, w: &Wrapper) -> Result<T> where
-        T: CRUDTable {
+        T: CRUDTable + DeserializeOwned {
         let mut conn = self.acquire().await?;
         conn.fetch_by_wrapper(w).await
     }
 
     async fn fetch_count_by_wrapper<T>(&self, w: &Wrapper) -> Result<u64> where
-        T: CRUDTable {
+        T: CRUDTable + DeserializeOwned {
         let mut conn = self.acquire().await?;
         conn.fetch_count_by_wrapper::<T>(w).await
     }
 
     async fn fetch_page_by_wrapper<T>(&self, w: &Wrapper, page: &dyn IPageRequest) -> Result<Page<T>> where
-        T: CRUDTable {
+        T: CRUDTable + DeserializeOwned {
         let mut conn = self.acquire().await?;
         conn.fetch_page_by_wrapper::<T>(w, page).await
     }
 
     async fn fetch_list<T>(&self) -> Result<Vec<T>> where
-        T: CRUDTable {
+        T: CRUDTable + DeserializeOwned {
         let mut conn = self.acquire().await?;
         conn.fetch_list().await
     }
 
-    async fn fetch_list_by_column<T, C>(&self, column: &str, column_values: &[C]) -> Result<Vec<T>> where
-        T: CRUDTable, C: Serialize + Send + Sync {
+    async fn fetch_list_by_column<T, P>(&self, column: &str, column_values: &[P]) -> Result<Vec<T>> where
+        T: CRUDTable + DeserializeOwned, P: Serialize + Send + Sync {
         if column_values.is_empty() {
             return Ok(vec![]);
         }
         let mut conn = self.acquire().await?;
-        conn.fetch_list_by_column::<T, C>(column, column_values).await
+        conn.fetch_list_by_column::<T, P>(column, column_values).await
     }
 
     async fn fetch_list_by_wrapper<T>(&self, w: &Wrapper) -> Result<Vec<T>> where
-        T: CRUDTable {
+        T: CRUDTable + DeserializeOwned {
         let mut conn = self.acquire().await?;
         conn.fetch_list_by_wrapper(w).await
     }
@@ -1061,7 +1048,7 @@ impl<T, P> Serialize for DynTableColumn<T, P> where T: CRUDTable, P: TableColumn
     }
 }
 
-impl<'de, T, P> Deserialize<'de> for DynTableColumn<T, P> where T: CRUDTable, P: TableColumnProvider {
+impl<'de, T, P> Deserialize<'de> for DynTableColumn<T, P> where T: CRUDTable + DeserializeOwned, P: TableColumnProvider {
     fn deserialize<D>(deserializer: D) -> std::result::Result<Self, <D as Deserializer<'de>>::Error> where
         D: Deserializer<'de> {
         let result = T::deserialize(deserializer)?;
@@ -1087,9 +1074,6 @@ impl<T, P> DerefMut for DynTableColumn<T, P> where T: CRUDTable, P: TableColumnP
 }
 
 impl<T, P> CRUDTable for DynTableColumn<T, P> where T: CRUDTable, P: TableColumnProvider {
-    /// is enable use plugin
-    fn is_use_plugin(plugin_name: &str) -> bool { T::is_use_plugin(plugin_name) }
-
     fn table_name() -> String {
         P::table_name()
     }
