@@ -91,7 +91,7 @@ impl<'a> ExecutorMut for $t {
             let q: DBQuery = self.bind_arg(&self.conn.driver_type, &sql, &args)?;
             result = self.conn.exec_prepare(q).await;
         } else {
-            result = self.conn.execute(&sql).await;
+            result = self.conn.exec(&sql).await;
         }
         if self.get_rbatis().log_plugin.is_enable() {
             match &result {
@@ -173,8 +173,8 @@ impl RbatisRef for $t {
 
 impl_executor!(RBatisConnExecutor<'_>);
 
-impl RBatisConnExecutor<'_> {
-    pub async fn begin(&mut self) -> crate::Result<RBatisTxExecutor<'_>> {
+impl<'a> RBatisConnExecutor<'a> {
+    pub async fn begin(self) -> crate::Result<RBatisTxExecutor<'a>> {
         let tx = self.conn.begin().await?;
         return Ok(RBatisTxExecutor {
             conn: tx,
@@ -185,7 +185,7 @@ impl RBatisConnExecutor<'_> {
 
 #[derive(Debug)]
 pub struct RBatisTxExecutor<'a> {
-    pub conn: DBTx<'a>,
+    pub conn: DBTx,
     pub rb: &'a Rbatis,
 }
 
@@ -198,22 +198,30 @@ impl<'a> RBatisTxExecutor<'a> {
 impl_executor!(RBatisTxExecutor<'_>);
 
 impl<'a> RBatisTxExecutor<'a> {
-    pub async fn commit(self) -> crate::Result<()> {
+    pub async fn begin(&mut self) -> crate::Result<()> {
+        return Ok(self.conn.begin().await?);
+    }
+    pub async fn commit(&mut self) -> crate::Result<()> {
         return Ok(self.conn.commit().await?);
     }
-    pub async fn rollback(self) -> crate::Result<()> {
+    pub async fn rollback(&mut self) -> crate::Result<()> {
         return Ok(self.conn.rollback().await?);
     }
-}
 
-impl <'a>Deref for RBatisTxExecutor<'a>{
-    type Target = DBTx<'a>;
-
-    fn deref(&self) -> &Self::Target {
-       &self.conn
+    pub fn take_conn(self) -> Option<DBPoolConn> {
+        return self.conn.take_conn();
     }
 }
-impl <'a> DerefMut for RBatisTxExecutor<'a>{
+
+impl<'a> Deref for RBatisTxExecutor<'a> {
+    type Target = DBTx;
+
+    fn deref(&self) -> &Self::Target {
+        &self.conn
+    }
+}
+
+impl<'a> DerefMut for RBatisTxExecutor<'a> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.conn
     }
@@ -229,6 +237,32 @@ impl<'a> RBatisTxExecutorGuard<'a> {
     pub fn as_executor(&'a mut self) -> RbatisExecutor<'a> {
         self.into()
     }
+
+    pub async fn begin(&mut self) -> crate::Result<()> {
+        let mut tx = self.tx.as_mut().ok_or_else(|| Error::from("[rbatis] tx is committed"))?;
+        return Ok(tx.begin().await?);
+    }
+
+    pub async fn commit(&mut self) -> crate::Result<()> {
+        let mut tx = self.tx.as_mut().ok_or_else(|| Error::from("[rbatis] tx is committed"))?;
+        return Ok(tx.commit().await?);
+    }
+
+    pub async fn rollback(&mut self) -> crate::Result<()> {
+        let mut tx = self.tx.as_mut().ok_or_else(|| Error::from("[rbatis] tx is committed"))?;
+        return Ok(tx.rollback().await?);
+    }
+
+    pub fn take_conn(mut self) -> Option<DBPoolConn> {
+        match self.tx.take() {
+            None => {
+                None
+            }
+            Some(s) => {
+                s.take_conn()
+            }
+        }
+    }
 }
 
 impl<'a> RBatisTxExecutor<'a> {
