@@ -37,6 +37,7 @@ use crate::core::Error;
 #[derive(Clone)]
 pub struct Wrapper {
     pub driver_type: DriverType,
+    pub dml: String,
     pub sql: String,
     pub args: Vec<Value>,
     pub formats: HashMap<String, fn(arg: &str) -> String>,
@@ -58,30 +59,24 @@ impl Debug for Wrapper {
             .field("driver_type", &self.driver_type)
             .field("sql", &self.sql)
             .field("args", &self.args)
+            .field("dml", &self.dml)
             //.field("formats", &formats)
             .finish()
     }
 }
 
 impl Wrapper {
+
     pub fn new(driver_type: &DriverType) -> Self {
         Self {
             driver_type: driver_type.clone(),
+            dml: "where".to_string(),
             sql: String::with_capacity(200),
             args: Vec::with_capacity(5),
             formats: Default::default(),
         }
     }
 
-    pub fn from(driver_type: &DriverType, sql: &str, args: Vec<Value>) -> Self {
-        Self {
-            driver_type: driver_type.clone(),
-            sql: sql.to_string(),
-            args: args,
-            formats: HashMap::new(),
-        }
-    }
-
     pub fn trim_value(mut self, from: &str, to: &str) -> Self {
         self.sql = self.sql.replace(from, to);
         self
@@ -92,6 +87,11 @@ impl Wrapper {
         self
     }
 
+    pub fn set_dml(mut self, dml:&str) ->Self{
+        self.dml = dml.to_string();
+        self
+    }
+
     /// link left Wrapper to this Wrapper
     /// for Example:
     /// let w = Wrapper::new(&DriverType::Postgres).push_sql("(").eq("a", "1").push_sql(")");
